/** INCLUDES *******************************************************/
#include <p33exxxx.h>
#include <adc.h>
#include <timer.h>
#include <pps.h>
#include <spi.h>
#include <uart.h>
#include <stdio.h>
#include "..\App1632_LCM.h"

#include "./USB/usb.h"
#include "HardwareProfile.h"
#include "./USB/usb_function_hid.h"

/** CONFIGURATION **************************************************/
/* Configuration Settings */
// OSC Input = HS w/PLL
// Debug Interface PGEC1, PGED1, Disable JTAG
// Disabld WDT, Clock Switch, Fail Safe Clock Monitor ,IO One Way.
// Power-On Reset 128ms, Enable BOR, Enable Alternate I2C1, I2C1.
// Disable Code-Protect, Write-Protect
// Disable Auxiliary Segment Code-protect, Write Protect.
_FOSCSEL( FNOSC_FRC & IESO_OFF ); 
_FICD( ICS_PGD1 & JTAGEN_OFF );
_FWDT( WDTPOST_PS32768 & WDTPRE_PR128 & PLLKEN_ON & WINDIS_OFF & FWDTEN_OFF );
_FOSC( POSCMD_HS & OSCIOFNC_OFF & IOL1WAY_OFF & FCKSM_CSECMD );
_FPOR( FPWRT_PWR128 & BOREN_ON & ALTI2C1_ON & ALTI2C2_ON );
_FGS( GWRP_OFF & GSS_OFF & GSSK_OFF );
_FAS( AWRP_OFF & APL_OFF & APLK_OFF );

#define SystemFrequency 30000000L
#define Timer1Tick ( ( SystemFrequency / 256 ) / Timer1TogglesPerSec )
#define Timer1TogglesPerSec 10
#define UART1Period	( ( SystemFrequency / UART1Baud ) / 16 ) - 1
#define UART1Baud 19200L
#define UART2Period	( ( SystemFrequency / UART2Baud ) / 16 ) - 1
#define UART2Baud 19200L
unsigned char UARTString1[ 32 ]; 
unsigned char Hours = 0 , Mins = 0 , Secs = 0;

#define DAC_CS LATDbits.LATD2
#define DAC_CS_TRIS TRISDbits.TRISD2
#define DAC_LDAC LATDbits.LATD1
#define DAC_LDAC_TRIS TRISDbits.TRISD1

/** VARIABLES ******************************************************/
const unsigned char LCMString1[ ] = "dsPIC33EPxxMU810"; 
const unsigned char LCMString2[ ] = "ChA:    ChB:    "; 
unsigned char Timer1TickFlag = 0 , DACDemoEnable = 1;
unsigned char ReceivedDataBuffer[ 64 ];
unsigned char ToSendDataBuffer[ 64 ];
USB_HANDLE USBOutHandle = 0;
USB_HANDLE USBInHandle = 0;
BOOL blinkStatusValid = TRUE;

/** PRIVATE PROTOTYPES *********************************************/
void BlinkUSBStatus(void);
BOOL Switch2IsPressed(void);
BOOL Switch3IsPressed(void);
static void InitializeSystem(void);
void ProcessIO(void);
void UserInit(void);
void YourHighPriorityISRCode();
void YourLowPriorityISRCode();
void USBCBSendResume(void);
WORD_VAL ReadPOT(void);
void SPI_DACUpdate( unsigned int ChAValue , unsigned int ChBValue );
void UART_PutRAMString( unsigned char * String );

void __attribute__((interrupt , no_auto_psv )) _T1Interrupt( void )
{
	IFS0bits.T1IF = 0;

	Timer1TickFlag++;
}

/** DECLARATIONS ***************************************************/
int main(void)
{   
    InitializeSystem();

    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif

    while(1)
    {
        #if defined(USB_POLLING)
	        USBDeviceTasks();
        #endif

        ProcessIO();        
    }
}

static void InitializeSystem(void)
{
	// FREF = Crystal / 2 = 8M Hz / 2 = 4M Hz
	CLKDIVbits.PLLPRE = 0;

	// FVOC = FREF * 60( 58 + 2 ) = 4M Hz * 60 = 240M Hz
	PLLFBDbits.PLLDIV = 58;
	
	// FOSC = FVOC / 4 = 240M Hz / 4 = 60M Hz
	CLKDIVbits.PLLPOST = 1;

	// Initiate Clock Switch to Primary Oscillator with PLL (NOSC = 0b011 )
	__builtin_write_OSCCONH( 0x03 );
	__builtin_write_OSCCONL( 0x01 );
	
	// Wait for Clock switch to occur
	while( OSCCONbits.COSC != 0b011 );
	
	// Wait for PLL to lock
	while( !OSCCONbits.LOCK );

	// Auxiliary PLL provides the source clock for auxiliary clock divider
	ACLKCON3bits.SELACLK = 1;

	// Auxiliary Oscillator Disabled
	ACLKCON3bits.AOSCMD = 0;

	// Primary Oscillator is the clock source for APLL
	ACLKCON3bits.ASRCSEL = 1;

	// Primary Oscillator is the clock source for APLL
	ACLKCON3bits.FRCSEL = 0;

	// AFREF = Crystal / 2 = 8M Hz / 2 = 4M Hz
	ACLKCON3bits.APLLPRE = 1;

	// FAVOC = FAREF * 24 = 4M Hz * 24 = 96M Hz
	ACLKDIV3bits.APLLDIV = 0x07;

	// FAOSC = FAVOC / 2 = 96Hz / 2 = 48M Hz
	ACLKCON3bits.APLLPOST = 0x06;

	// Enable Auxiliary PLL (APLL) and Select APLL as USB Clock Source bit
	ACLKCON3bits.ENAPLL = 1;

	// Wait for PLL to lock
   	while( !ACLKCON3bits.APLLCK ); 

	#if defined(USE_USB_BUS_SENSE_IO)
		tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif

	#if defined(USE_SELF_POWER_SENSE_IO)
		tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif

   	ANSELA = 0x0000;
	ANSELB = 0x0000;
	ANSELC = 0x0000;
	ANSELD = 0x0000;
	ANSELE = 0x0000;
	ANSELG = 0x0000;

    UserInit();
 
    USBDeviceInit();
}

void UserInit(void)
{
    //Initialize all of the LED pins
    mInitAllLEDs();
    
    //Initialize all of the push buttons
    mInitAllSwitches();
    
	mInitPOT();

	OpenTimer1( T1_ON & T1_IDLE_CON & T1_GATE_OFF & T1_PS_1_256 & T1_SYNC_EXT_OFF & T1_SOURCE_INT , Timer1Tick );
	ConfigIntTimer1( T1_INT_ON & T1_INT_PRIOR_4 );

	LCM_Init( );

	LCM_SetCursor( 0 , 0 );
	LCM_PutROMString( LCMString1 );
	LCM_SetCursor( 0 , 1 );
	LCM_PutROMString( LCMString2 );

	PPSUnLock;								// __builtin Function
	iPPSInput( IN_FN_PPS_U1RX , IN_PIN_PPS_RP98 );
	iPPSOutput( OUT_PIN_PPS_RP104 , OUT_FN_PPS_U1TX );
	PPSLock;								// __builtin Function

	OpenUART1( UART_EN & UART_IDLE_CON & UART_IrDA_DISABLE & UART_MODE_SIMPLEX & UART_UEN_00 & 
			   UART_EN_WAKE & UART_DIS_LOOPBACK & UART_DIS_ABAUD & UART_UXRX_IDLE_ONE & 
			   UART_NO_PAR_8BIT & UART_BRGH_SIXTEEN & UART_1STOPBIT ,
			   UART_INT_TX & UART_IrDA_POL_INV_ZERO & UART_SYNC_BREAK_DISABLED & UART_TX_ENABLE &
			   UART_INT_RX_CHAR & UART_ADR_DETECT_DIS & UART_RX_OVERRUN_CLEAR ,
			   UART1Period );

	PPSUnLock;								// __builtin Function
	iPPSInput( IN_FN_PPS_U2RX , IN_PIN_PPS_RP100 );
	iPPSOutput( OUT_PIN_PPS_RP101 , OUT_FN_PPS_U2TX );
	PPSLock;								// __builtin Function

	OpenUART2( UART_EN & UART_IDLE_CON & UART_IrDA_DISABLE & UART_MODE_SIMPLEX & UART_UEN_00 & 
			   UART_EN_WAKE & UART_DIS_LOOPBACK & UART_DIS_ABAUD & UART_UXRX_IDLE_ONE & 
			   UART_NO_PAR_8BIT & UART_BRGH_SIXTEEN & UART_1STOPBIT ,
			   UART_INT_TX & UART_IrDA_POL_INV_ZERO & UART_SYNC_BREAK_DISABLED & UART_TX_ENABLE &
			   UART_INT_RX_CHAR & UART_ADR_DETECT_DIS & UART_RX_OVERRUN_CLEAR ,
			   UART2Period );

	PPSUnLock;								// __builtin Function
	iPPSInput( IN_FN_PPS_SDI1 , IN_PIN_PPS_RP124 );
	iPPSOutput( OUT_PIN_PPS_RP112 , OUT_FN_PPS_SCK1 );
	iPPSOutput( OUT_PIN_PPS_RP113 , OUT_FN_PPS_SDO1 );
	PPSLock;								// __builtin Function

	DAC_CS_TRIS = 0;
	DAC_CS = 1;
	DAC_LDAC_TRIS = 0;
	DAC_LDAC = 1;

	OpenSPI1( ENABLE_SCK_PIN & ENABLE_SDO_PIN & SPI_MODE16_OFF & SPI_SMP_OFF & SPI_CKE_ON & SLAVE_ENABLE_OFF &
			  CLK_POL_ACTIVE_HIGH & MASTER_ENABLE_ON & SEC_PRESCAL_8_1 & PRI_PRESCAL_64_1 ,
			  FRAME_ENABLE_OFF & FRAME_SYNC_OUTPUT & FRAME_POL_ACTIVE_LOW & 
			  FRAME_SYNC_EDGE_PRECEDE & FIFO_BUFFER_DISABLE ,
			  SPI_ENABLE & SPI_IDLE_CON & SPI_RX_OVFLOW_CLR & 
			  BUF_ELE_COUNT_0 & BUF_INT_SEL_0 & SPI_SHFTREG_NOT_EMPTY & RX_FIFO_NOT_EMPTY );

   //initialize the variable holding the handle for the last
    // transmission
    USBOutHandle = 0;
    USBInHandle = 0;

    blinkStatusValid = TRUE;
}

void ProcessIO(void)
{   
	unsigned int Temporary = 0;
	static unsigned int DACChAValue = 0 , DACChBValue = 0;
	static unsigned char TimerIndex = 0;

	//Blink the LEDs according to the USB device status
    if(blinkStatusValid)
        BlinkUSBStatus();

	if( Timer1TickFlag > 0 )
	{
		Timer1TickFlag--;
	
		SPI_DACUpdate( DACChAValue , DACChBValue );

		LCM_SetCursor( 4 , 1 );
		LCM_PutNumber( DACChAValue , 4 );
		LCM_SetCursor( 12 , 1 );
		LCM_PutNumber( DACChBValue , 4 );
	
		if( DACDemoEnable )
		{
			if( ( DACChAValue += 40 ) > 2399 )
				DACChAValue = 0;
		}
	
		if( ( DACChBValue += 40 ) > 4095 )
			DACChBValue = 0;

		if( ++TimerIndex > 9 )
		{
			TimerIndex = 0;

			if( ++Secs > 59 ) 
			{
				Secs = 0;
				if( ++Mins > 59 )
				{
					Mins = 0;
					if( ++Hours > 23 )
						Hours = 0;
				}
			}
	
			sprintf( ( char * ) UARTString1 , "Now Time %2d:%2d:%2d\r" ,
					 Hours , Mins , Secs );
	
			UART_PutRAMString( UARTString1 );
		}
	}

    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;
    
	if(!HIDTxHandleBusy(USBInHandle))
	{
		ToSendDataBuffer[ 0 ] = 0x37;		//Echo back to the host the command we are fulfilling in the first byte.  In this case, the Read POT (analog voltage) command.

		// Switch
		ToSendDataBuffer[ 1 ] = ( ( !sw3 << 3 ) | ( !Boardsw5 << 2 ) | ( !Boardsw4 << 1 ) | ( !sw2 ) );

		// VR1
		Temporary =ReadADC1( 3 );
		ToSendDataBuffer[ 2 ] = Temporary & 0xff;
		ToSendDataBuffer[ 3 ] = ( Temporary >> 8 ) & 0xff;

		// VR1Board
		Temporary =ReadADC1( 0 );
		ToSendDataBuffer[ 4 ] = Temporary & 0xff;
		ToSendDataBuffer[ 5 ] = ( Temporary >> 8 ) & 0xff;

		// VR2Board
		Temporary =ReadADC1( 2 );
		ToSendDataBuffer[ 6 ] = Temporary & 0xff;
		ToSendDataBuffer[ 7 ] = ( Temporary >> 8 ) & 0xff;

		// Temperature
		Temporary = ReadADC1( 1 );
		ToSendDataBuffer[ 8 ] = Temporary & 0xff;
		ToSendDataBuffer[ 9 ] = ( Temporary >> 8 ) & 0xff;

		// DAC
		ToSendDataBuffer[ 10 ] = DACChAValue & 0xff;
		ToSendDataBuffer[ 11 ] = ( DACChAValue >> 8 ) & 0xff;

		USBInHandle = HIDTxPacket( HID_EP ,(BYTE * ) &ToSendDataBuffer[ 0 ] , 64 );
	}

    if(!HIDRxHandleBusy(USBOutHandle))				//Check if data was received from the host.
    {   
        switch(ReceivedDataBuffer[0])				//Look at the data the host sent, to see what kind of application specific command it sent.
        {
            case 0x80:  //Toggle LEDs command
		        blinkStatusValid = FALSE;			//Stop blinking the LEDs automatically, going to manually control them now.
                if(mGetLED_1() == mGetLED_2())
                {
                    mLED_1_Toggle();
                    mLED_2_Toggle();
                }

                else
                {
                    if(mGetLED_1())
					{
                        mLED_2_On();
					}
                    else
                    {
					    mLED_2_Off();
	               	 }
				}
                break;

            case 0x90:  // DAC command
				DACDemoEnable = 0;
				DACChAValue = ( ReceivedDataBuffer[ 2 ] << 8 ) + ReceivedDataBuffer[ 1 ];
				break;

        }
        //Re-arm the OUT endpoint for the next packet
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
    }
}

void BlinkUSBStatus(void)
{
    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    static WORD led_count = 0;
    
    if( led_count == 0 )
		led_count = 20000L;

	led_count--;

	if( USBSuspendControl == 1 )
    {
        if( led_count == 0 )
        {
            mLED_1_Toggle();
            if(mGetLED_1())
			{
                mLED_2_On();
			}
            else
			{
                mLED_2_Off();
			}
        }
	}
    else
    {
        if( USBDeviceState == DETACHED_STATE )
		{
			mLED_Both_Off();
		}

        else if( USBDeviceState == ATTACHED_STATE )
		{
			mLED_Both_On();
		}

        else if( USBDeviceState == POWERED_STATE )
		{
			mLED_Only_1_On();
		}

		else if( USBDeviceState == DEFAULT_STATE )
		{
			mLED_Only_2_On();
		}

        else if( USBDeviceState == ADDRESS_STATE )
        {
 			if( led_count == 0 )
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }
        }

		else if( USBDeviceState == CONFIGURED_STATE )
        {
           if( led_count == 0 )
           {      
                mLED_1_Toggle();         
                if(mGetLED_1())
				{
                    mLED_2_Off();
				}
                else
				{
                    mLED_2_On();
				}
           }
        }
    }
}

void USBCBSuspend(void)
{
	_IPL = 1;
    USBSleepOnSuspend();
}

void USBCBWakeFromSuspend(void)
{
}


void USBCB_SOF_Handler(void)
{
}


void USBCBErrorHandler(void)
{
}

void USBCBCheckOtherReq(void)
{
	USBCheckHIDRequest();
}

void USBCBStdSetDscHandler(void)
{
}

void USBCBInitEP(void)
{
    //enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);
    //Re-arm the OUT endpoint for the next packet
    USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
}

void USBCBSendResume(void)
{
    static WORD delay_count;
    
    //First verify that the host has armed us to perform remote wakeup.
    //It does this by sending a SET_FEATURE request to enable remote wakeup,
    //usually just before the host goes to standby mode (note: it will only
    //send this SET_FEATURE request if the configuration descriptor declares
    //the device as remote wakeup capable, AND, if the feature is enabled
    //on the host (ex: on Windows based hosts, in the device manager 
    //properties page for the USB device, power management tab, the 
    //"Allow this device to bring the computer out of standby." checkbox 
    //should be checked).
    if(USBGetRemoteWakeupStatus() == TRUE) 
    {
        //Verify that the USB bus is in fact suspended, before we send
        //remote wakeup signalling.
        if(USBIsBusSuspended() == TRUE)
        {
            USBMaskInterrupts();
            
            //Clock switch to settings consistent with normal USB operation.
            USBCBWakeFromSuspend();
            USBSuspendControl = 0; 
            USBBusIsSuspended = FALSE;  //So we don't execute this code again, 
                                        //until a new suspend condition is detected.

            //Section 7.1.7.7 of the USB 2.0 specifications indicates a USB
            //device must continuously see 5ms+ of idle on the bus, before it sends
            //remote wakeup signalling.  One way to be certain that this parameter
            //gets met, is to add a 2ms+ blocking delay here (2ms plus at 
            //least 3ms from bus idle to USBIsBusSuspended() == TRUE, yeilds
            //5ms+ total delay since start of idle).
            delay_count = 3600U;        
            do
            {
                delay_count--;
            }while(delay_count);
            
            //Now drive the resume K-state signalling onto the USB bus.
            USBResumeControl = 1;       // Start RESUME signaling
            delay_count = 1800U;        // Set RESUME line for 1-13 ms
            do
            {
                delay_count--;
            }while(delay_count);
            USBResumeControl = 0;       //Finished driving resume signalling

            USBUnmaskInterrupts();
        }
    }
}

BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_TRANSFER:
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER_TERMINATED:
           break;
        default:
            break;
    }      
    return TRUE; 
}

void SPI_DACUpdate( unsigned int ChAValue , unsigned int ChBValue )
{
	unsigned long i = 0;

	// CS Active
	DAC_CS = 0;

	while( SPI1STATbits.SPITBF );
	WriteSPI1( 0x70 | ( ( ChAValue >> 8 ) & 0x000f ) );
	while( !DataRdySPI1( ) );
	ReadSPI1( );							//Dummy Read

	while( SPI1STATbits.SPITBF );
	WriteSPI1( ChAValue & 0x00ff );
	while( !DataRdySPI1( ) );
	ReadSPI1( );							//Dummy Read

	// Wait For Last Byte Transmitted
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	DAC_CS = 1;

	// CS Active
	DAC_CS = 0;

	while( SPI1STATbits.SPITBF );
	WriteSPI1( 0xf0 | ( ( ChBValue >> 8 ) & 0x000f ) );
	while( !DataRdySPI1( ) );
	ReadSPI1( );							//Dummy Read

	while( SPI1STATbits.SPITBF );
	WriteSPI1( ChBValue& 0x00ff );
	while( !DataRdySPI1( ) );
	ReadSPI1( );							//Dummy Read

	// Wait For Last Byte Transmitted
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	DAC_CS = 1;

	// CS Active
	DAC_LDAC = 0;
	for( i = 0; i < 10 ; i++ );

	// CS Inactive
	DAC_LDAC = 1;
}

void UART_PutRAMString( unsigned char * String ) 
{
	while( *String != 0x00 )
	{
		while( BusyUART1( ) );
	    WriteUART1( *String );
		while( BusyUART2( ) );
	    WriteUART2( *String );
		String++;
	}
}

